@@ -1,11 +1,12 @@
 import asyncio
 from typing import Dict, Set
+from warnings import warn
 
 from discord import User
 from discord.ext.commands import Context
 
 from dpymenus import ButtonMenu
-from dpymenus.exceptions import ButtonsError, PagesError
+from dpymenus.exceptions import ButtonsError, CallbackError, PagesError
 
 
 class Poll(ButtonMenu):
@@ -15,7 +16,7 @@ class Poll(ButtonMenu):
     It contains Page objects which represent new Menu states that call methods for validation and handling.
 
     Attributes:
-        ctx: A reference to the command Context.
+        ctx: A reference to the command context.
         timeout: How long (in seconds) before the poll ends.
         voted: Dictionary that tracks users who have already voted.
     """
@@ -25,16 +26,16 @@ class Poll(ButtonMenu):
         self.voted: Set[User] = set()
 
     def __repr__(self):
-        return f'<Menu pages={[p.__str__() for p in self.pages]}, timeout={self.timeout}, active={self.active} page={self.page},' \
-               f'state_fields={self.state_fields}>'
+        return f'<Menu pages={[p.__str__() for p in self.pages]}, timeout={self.timeout}, active={self.active} page={self.page_index},' \
+               f'data={self.data}>'
 
     async def open(self):
         """The entry point to a new Menu instance. This will start a loop until a Page object with None as its function is set.
         Manages gathering user input, basic validation, sending messages, and cancellation requests."""
-        self._validate_pages()
-        self._set_state_fields()
+        await self._validate_pages()
+        await self._set_data()
 
-        self.output = await self.ctx.send(embed=self.pages[self.page])
+        self.output = await self.ctx.send(embed=self.page)
         await self._add_buttons()
 
         pending = set()
@@ -51,21 +52,21 @@ class Poll(ButtonMenu):
     # Utility Methods
     async def results(self) -> Dict[str, int]:
         """Utility method to get a dictionary of poll results."""
-        return {k: len(v) for k, v in self.state_fields.items()}
+        return {k: len(v) for k, v in self.data.items()}
 
     async def add_results_fields(self):
         """Utility method to add new fields to your next page automatically."""
-        for k, v in self.state_fields.items():
-            self.pages[self.page + 1].add_field(name=k, value=str(len(v)))
+        for k, v in self.data.items():
+            self.pages[self.page_index + 1].add_field(name=k, value=str(len(v)))
 
     async def generate_results_page(self):
         """Utility method to build your entire results page automatically."""
-        next_page = self.pages[self.page + 1]
+        next_page = self.pages[self.page_index + 1]
 
         await self.add_results_fields()
 
-        highest_value = max(self.state_fields.values())
-        winning_key = {key for key, value in self.state_fields.items() if value == highest_value}
+        highest_value = max(self.data.values())
+        winning_key = {key for key, value in self.data.items() if value == highest_value}
 
         if len(highest_value) == 0:
             next_page.description = ' '.join([next_page.description, f"It's a draw!"])
@@ -84,8 +85,8 @@ class Poll(ButtonMenu):
                 return
 
             else:
-                if reaction.emoji in self.pages[self.page].buttons:
-                    self.state_fields[reaction.emoji].add(user.id)
+                if reaction.emoji in self.page.buttons:
+                    self.data[reaction.emoji].add(user.id)
 
     async def _get_vote_remove(self):
         """Watches for a user removing a reaction on the Poll. Removes them from the relevant state_field values."""
@@ -97,33 +98,34 @@ class Poll(ButtonMenu):
                 return
 
             else:
-                if reaction.emoji in self.pages[self.page].buttons:
-                    self.state_fields[reaction.emoji].remove(user.id)
+                if reaction.emoji in self.page.buttons:
+                    self.data[reaction.emoji].remove(user.id)
 
     async def _poll_timer(self):
         """Handles poll duration."""
         await asyncio.sleep(self.timeout)
 
     async def _finish_poll(self):
-        """Removes multi-votes and calls the Page callback function when finished."""
+        """Removes multi-votes and calls the Page on_next function when finished."""
         check_cheaters = False
-        for value in self.state_fields.values():
+        for value in self.data.values():
             if value:
                 check_cheaters = True
                 break
 
         if check_cheaters:
             cheaters = await self._get_cheaters()
-            for voter_set in self.state_fields.values():
+            for voter_set in self.data.values():
                 voter_set -= cheaters
 
-        await self.pages[self.page].callback(self)
+        await self.output.clear_reactions()
+        await self.page.on_next(self)
 
     async def _get_cheaters(self) -> Set[int]:
         """Returns a set of user ID's that appear in more than one state_field value."""
         seen = set()
         repeated = set()
-        for voter_set in self.state_fields.values():
+        for voter_set in self.data.values():
             for voter in voter_set:
                 if voter in seen:
                     repeated.add(voter)
@@ -132,23 +134,28 @@ class Poll(ButtonMenu):
 
         return repeated
 
-    def _set_state_fields(self):
-        """Internally sets state field keys and values based on the current Page button properties."""
-        self._validate_buttons()
+    async def _set_data(self):
+        """Internally sets data field keys and values based on the current Page button properties."""
+        await self._validate_buttons()
 
-        print(self.pages[0].buttons)
-        for button in self.pages[self.page].buttons:
-            self.state_fields.update({button: set()})
+        for button in self.page.buttons:
+            self.data.update({button: set()})
 
-    def _validate_buttons(self):
+    async def _validate_buttons(self):
         """Checks that Poll objects always have more than two buttons."""
-        if len(self.pages[0].buttons) < 2:
-            raise ButtonsError('A Poll primary page must have at least two buttons.')
+        if len(self.page.buttons) < 2:
+            raise ButtonsError(f'A Poll primary page must have at least two buttons. Expected at least 2, found {len(self.page.buttons)}.')
 
-    def _validate_pages(self):
+    async def _validate_pages(self):
         """Checks that the Menu contains at least one Page."""
         if len(self.pages) != 2:
-            raise PagesError('A Poll can only have two pages.')
+            raise PagesError(f'A Poll can only have two pages. Expected 2, found {len(self.pages)}.')
+
+        if self.page.on_cancel or self.page.on_fail or self.page.on_timeout:
+            raise CallbackError('A Poll can not have `on_cancel`, `on_fail`, or `on_timeout` callbacks.')
+
+        if len(self.page.buttons) > 5:
+            warn('Adding more than 5 buttons to a page at once may result in discord.py throttling the bot client.')
 
     @staticmethod
     async def get_voters(users: Set[User]) -> Set[User]:
